@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2021  The R Core Team.
+ *  Copyright (C) 1998-2023  The R Core Team.
  *  Copyright (C) 1995-1998  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -138,31 +138,27 @@ static SEXP seq_colon(double n1, double n2, SEXP call)
     return ans;
 }
 
-SEXP attribute_hidden do_colon(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_colon(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP s1, s2;
-    double n1, n2;
-
     checkArity(op, args);
-    if (inherits(CAR(args), "factor") && inherits(CADR(args), "factor"))
-	return(cross_colon(call, CAR(args), CADR(args)));
+    SEXP s1 = CAR(args),
+	s2 = CADR(args);
+    if (inherits(s1, "factor") && inherits(s2, "factor"))
+	return(cross_colon(call, s1, s2));
+    double
+	n1 = length(s1),
+	n2 = length(s2);
+    if (n1 != 1 || n2 != 1) {
+	if (n1 == 0 || n2 == 0)
+	    errorcall(call, _("argument of length 0"));
+	char *check = getenv("_R_CHECK_LENGTH_COLON_");
+	if (check ? StringTrue(check) : FALSE) // warn by default
+	    errorcall(call, _("numerical expression has length > 1"));
+	else
+	    warningcall(call, _("numerical expression has %d elements: only the first used"),
+			(n1 > 1) ? (int) n1 : (int) n2);
+    }
 
-    s1 = CAR(args);
-    s2 = CADR(args);
-    n1 = length(s1);
-    n2 = length(s2);
-    if (n1 == 0 || n2 == 0)
-	errorcall(call, _("argument of length 0"));
-    if (n1 > 1)
-	warningcall(call,
-		    ngettext("numerical expression has %d element: only the first used",
-			     "numerical expression has %d elements: only the first used",
-			     (int) n1), (int) n1);
-    if (n2 > 1)
-	warningcall(call,
-		    ngettext("numerical expression has %d element: only the first used",
-			     "numerical expression has %d elements: only the first used",
-			     (int) n2), (int) n2);
     n1 = asReal(s1);
     n2 = asReal(s2);
     if (ISNAN(n1) || ISNAN(n2))
@@ -339,7 +335,7 @@ static SEXP rep3(SEXP s, R_xlen_t ns, R_xlen_t na)
 }
 
 // .Internal(rep.int(x, times))
-SEXP attribute_hidden do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     SEXP s = CAR(args), ncopy = CADR(args);
@@ -403,7 +399,7 @@ SEXP attribute_hidden do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
     return a;
 }
 
-SEXP attribute_hidden do_rep_len(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_rep_len(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     R_xlen_t ns, na;
     SEXP a, s, len;
@@ -633,7 +629,7 @@ static SEXP rep4(SEXP x, SEXP times, R_xlen_t len, R_xlen_t each, R_xlen_t nt)
    rep(1:3,,8) matches length.out */
 
 /* This is a primitive SPECIALSXP with internal argument matching */
-SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x, times = R_NilValue;
     R_xlen_t i, lx, len = NA_INTEGER, each = 1, nt;
@@ -798,7 +794,7 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
  */
 #define FEPS 1e-10
 /* to match seq.default */
-SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans = R_NilValue /* -Wall */, from, to, by, len, along;
     R_xlen_t i, lout = NA_INTEGER;
@@ -810,7 +806,7 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     /* This is a primitive and we manage argument matching ourselves.
        We pretend this is
-       seq(from, to, by, length.out, along.with, ...)
+       seq.int(from, to, by, length.out, along.with, ...)
     */
     static SEXP do_seq_formals = NULL;
     if (do_seq_formals == NULL)
@@ -849,12 +845,17 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
     } else if(len != R_MissingArg && len != R_NilValue) {
 	double rout = asReal(len);
+	if(!R_FINITE(rout))
+	    errorcall(call, _("'%s' must be a finite number"), "length.out");
 	if(ISNAN(rout) || rout <= -0.5)
 	    errorcall(call, _("'length.out' must be a non-negative number"));
 	if(length(len) != 1)
 	    warningcall(call, _("first element used of '%s' argument"),
 			"length.out");
-	lout = (R_xlen_t) ceil(rout);
+	rout = ceil(rout);
+	if(rout >= R_XLEN_T_MAX)
+	    errorcall(call, _("result would be too long a vector"));
+	lout = (R_xlen_t) rout;
     }
 
     if(lout == NA_INTEGER) {
@@ -1047,7 +1048,7 @@ done:
     return ans;
 }
 
-SEXP attribute_hidden do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
     R_xlen_t len;
@@ -1083,7 +1084,7 @@ SEXP attribute_hidden do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
 	return R_compact_intrange(1, len);
 }
 
-SEXP attribute_hidden do_seq_len(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_seq_len(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     R_xlen_t len;
 
@@ -1112,7 +1113,7 @@ SEXP attribute_hidden do_seq_len(SEXP call, SEXP op, SEXP args, SEXP rho)
 	return R_compact_intrange(1, len);
 }
 
-SEXP attribute_hidden do_sequence(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_sequence(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     R_xlen_t lengths_len, from_len, by_len, ans_len, i, i2, i3;
     int from_elt, by_elt, length, j, k, *ans_elt;
