@@ -1014,7 +1014,7 @@ attribute_hidden SEXP do_range(SEXP call, SEXP op, SEXP args, SEXP env)
     PROTECT(op = findFun(install("range.default"), env));
     PROTECT(prargs = promiseArgs(args, R_GlobalEnv));
     for (a = args, b = prargs; a != R_NilValue; a = CDR(a), b = CDR(b))
-	SET_PRVALUE(CAR(b), CAR(a));
+	IF_PROMSXP_SET_PRVALUE(CAR(b), CAR(a));
     ans = applyClosure(call, op, prargs, env, R_NilValue);
 #ifdef ADJUST_ENVIR_REFCNTS
     unpromiseArgs(prargs);
