@@ -579,6 +579,14 @@ write_one (unsigned int namescount, const char * const *names, void *data)
 }
 #endif
 
+// Copied from platform.c (only used condiitonally there)
+// case-insensitive string comparison
+int static R_strieql(const char *a, const char *b)
+{
+    while (*a && *b && toupper(*a) == toupper(*b)) { a++; b++; }
+    return (*a == 0 && *b == 0);
+}
+
 #include "RBufferUtils.h"
 
 /* iconv(x, from, to, sub, mark) */
@@ -590,7 +598,7 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
     void * utf8_obj = (iconv_t)-1;
     const char *inbuf;
     char *outbuf;
-    const char *sub;
+    const char *sub; // null for no substitution.
     size_t inb, outb, res;
     R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
     Rboolean isRawlist = FALSE;
@@ -632,12 +640,12 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 	toRaw = asLogical(CAR(args));
 	if(toRaw == NA_LOGICAL)
 	    error(_("invalid '%s' argument"), "toRaw");
-	/* some iconv's allow "UTF8", but libiconv does not */
-	if(streql(from, "UTF8") || streql(from, "utf8") ) from = "UTF-8";
-	if(streql(to, "UTF8") || streql(to, "utf8") ) to = "UTF-8";
+	/* some iconv's allow "UTF8", but GNU libiconv does not */
+	if(R_strieql(from, "UTF8")) from = "UTF-8";
+	if(R_strieql(to, "UTF8")) to = "UTF-8";
 	if(streql(to, "UTF-8")) isUTF8 = TRUE;
-	if(streql(to, "latin1") || streql(to, "ISO_8859-1")
-	    || streql(to, "CP1252")) isLatin1 = TRUE;
+	if(R_strieql(to, "latin1") || R_strieql(to, "ISO_8859-1")
+	    || R_strieql(to, "CP1252")) isLatin1 = TRUE;
 	if(streql(to, "") && known_to_be_latin1) isLatin1 = TRUE;
 	if(streql(to, "") && known_to_be_utf8) isUTF8 = TRUE;
 	isRawlist = (TYPEOF(x) == VECSXP);
@@ -689,8 +697,19 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 			error(_("unsupported conversion from '%s' to '%s' in codepage %d"),
 			      "UTF-8", to, localeCP);
 		#else
-			error(_("unsupported conversion from '%s' to '%s'"),
-			      "UTF-8", to);
+		    {
+			// musl does not support ASCII//TRANSLIT but has
+			// similar ASCII subsituting with *
+			// In case there are others, we set sub here.
+			if(streql(to, "ASCII//TRANSLIT")) {
+			    to = "ASCII";
+			    utf8_obj = Riconv_open(to, "UTF-8");
+			    if(!sub) sub = "c99";
+			}
+			if(utf8_obj == (iconv_t)(-1))
+			    error(_("unsupported conversion from '%s' to '%s'"),
+				  "UTF-8", to);
+		    }
 		#endif
 		}
 		obj = utf8_obj;
@@ -703,8 +722,16 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 			error(_("unsupported conversion from '%s' to '%s' in codepage %d"),
 			      "latin1", to, localeCP);
 		#else
-			error(_("unsupported conversion from '%s' to '%s'"),
-			      "latin1", to);
+		    {
+			if(streql(to, "ASCII//TRANSLIT")) {
+			    to = "ASCII";
+			    latin1_obj = Riconv_open(to, "latin1");
+			    if(!sub) sub = "?";
+			}
+			if(latin1_obj == (iconv_t)(-1))
+			    error(_("unsupported conversion from '%s' to '%s'"),
+				  "latin1", to);			   
+		    }
 		#endif
 		}
 		obj = latin1_obj;
@@ -716,8 +743,16 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 			error(_("unsupported conversion from '%s' to '%s' in codepage %d"),
 			      from, to, localeCP);
 		#else
-			error(_("unsupported conversion from '%s' to '%s'"),
-			      from, to);
+		    {
+			if(streql(to, "ASCII//TRANSLIT")) {
+			    to = "ASCII";
+			    arg_obj = Riconv_open(to, from);
+			    if(!sub) sub = "?";
+			}
+			if(arg_obj == (iconv_t)(-1))
+			    error(_("unsupported conversion from '%s' to '%s'"),
+				  from, to);			   
+		    }
 		#endif
 		}
 		obj = arg_obj;
@@ -759,7 +794,7 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 			    ucs = (R_wchar_t) wc;
 			inbuf += clen; inb -= clen;
 			if(ucs < 65536) {
-			    // gcc 7 objects to this with unsigned int
+			    // gcc 7 objected to this with unsigned int
 			    snprintf(outbuf, 9, "<U+%04X>", (unsigned short) ucs);
 			    outbuf += 8; outb -= 8;
 			} else {
@@ -787,7 +822,7 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 			    ucs = (R_wchar_t) wc;
 			inbuf += clen; inb -= clen;
 			if(ucs < 65536) {
-			    // gcc 7 objects to this with unsigned int
+			    // gcc 7 objected to this with unsigned int
 			    snprintf(outbuf, 7, "\\u%04x", (unsigned short) ucs);
 			    outbuf += 6; outb -= 6;
 			} else {
