@@ -14,9 +14,9 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA. */
+   along with this program; if not, a copy is available at
+   http://www.r-project.org/Licenses/
+*/
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -24,6 +24,9 @@
 #include <ctype.h>
 #include <errno.h>
 #include <math.h>
+#include <float.h>
+#include <limits.h>
+#include <string.h>
 #include "foreign.h"
 #include "avl.h"
 #include "file-handle.h"
@@ -40,7 +43,7 @@
 	((A) < (B) ? (B) : ((A) > (C) ? (C) : (A)))
 
 /* Divides nonnegative X by positive Y, rounding up. */
-#define DIV_RND_UP(X, Y) 			\
+#define DIV_RND_UP(X, Y)			\
 	(((X) + ((Y) - 1)) / (Y))
 
 /* Returns nonnegative difference between {nonnegative X} and {the
@@ -52,47 +55,21 @@
 	  rem ? (Y) - rem : 0;			\
 	})
 #else
-#define REM_RND_UP(X, Y) 			\
+#define REM_RND_UP(X, Y)			\
 	((X) % (Y) ? (Y) - (X) % (Y) : 0)
 #endif
 
 /* Rounds X up to the next multiple of Y. */
-#define ROUND_UP(X, Y) 				\
+#define ROUND_UP(X, Y)				\
 	(((X) + ((Y) - 1)) / (Y) * (Y))
 
 /* Rounds X down to the previous multiple of Y. */
-#define ROUND_DOWN(X, Y) 			\
+#define ROUND_DOWN(X, Y)			\
 	((X) / (Y) * (Y))
 
 #undef DEBUGGING
 /*#define DEBUGGING 1*/
 
-static double 
-second_lowest_double_val()
-{
-  /* PORTME: Set the value for second_lowest_value, which is the
-     "second lowest" possible value for a double.  This is the value
-     for LOWEST on MISSING VALUES, etc. */
-#if FPREP == FPREP_IEEE754
-#ifdef WORDS_BIGENDIAN
-    union {
-	unsigned char c[8];
-	double d;
-    } second_lowest = {{0xff, 0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe}};
-#else
-    union {
-	unsigned char c[8];
-	double d;
-    } second_lowest = {{0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xff}};
-#endif
-    return second_lowest.d;
-#else /* FPREP != FPREP_IEEE754 */
-#error Unknown floating-point representation.
-#endif /* FPREP != FPREP_IEEE754 */
-}
-
-#define SECOND_LOWEST_VALUE second_lowest_double_val()
-#define NOT_INT INT_MIN
 
 /* pfm's file_handle extension. */
 struct pfm_fhuser_ext
@@ -134,8 +111,8 @@ pfm_close (struct file_handle * h)
 {
   struct pfm_fhuser_ext *ext = h->ext;
 
-  Free (ext->vars);
-  Free (ext->trans);
+  R_Free (ext->vars);
+  R_Free (ext->trans);
   if (EOF == fclose (ext->file))
     error(_("%s: Closing portable file: %s"), h->fn, strerror (errno));
 }
@@ -163,7 +140,7 @@ fill_buf (struct file_handle *h)
   /* PORTME: line ends. */
   {
     int c;
-    
+
     c = getc (ext->file);
     if (c != '\n' && c != '\r')
       lose ((_("Bad line end")));
@@ -172,11 +149,11 @@ fill_buf (struct file_handle *h)
     if (c != '\n' && c != '\r')
       ungetc (c, ext->file);
   }
-  
+
   if (ext->trans)
     {
       int i;
-      
+
       for (i = 0; i < 80; i++)
 	ext->buf[i] = ext->trans[ext->buf[i]];
     }
@@ -210,7 +187,7 @@ static inline int
 skip_char (struct file_handle *h, int c)
 {
   struct pfm_fhuser_ext *ext = h->ext;
-  
+
   if (ext->cc == c)
     {
       advance ();
@@ -258,12 +235,12 @@ pfm_read_dictionary (struct file_handle *h, struct pfm_read_info *inf)
 #endif
 
   /* Open the physical disk file. */
-  ext = (struct pfm_fhuser_ext *) Calloc(1, struct pfm_fhuser_ext);
+  ext = (struct pfm_fhuser_ext *) R_Calloc(1, struct pfm_fhuser_ext);
   ext->file = fopen (R_ExpandFileName(h->norm_fn), "rb");
   if (ext->file == NULL)
     {
-      Free (ext);
-      error(_("an error occurred while opening \"%s\" for reading as a portable file: %s"), 
+      R_Free (ext);
+      error(_("an error occurred while opening \"%s\" for reading as a portable file: %s"),
 	    h->fn, strerror (errno));
       return NULL;
     }
@@ -280,7 +257,7 @@ pfm_read_dictionary (struct file_handle *h, struct pfm_read_info *inf)
   /* Read the header. */
   if (!read_header (h))
     goto lossage;
-  
+
   /* Read version, date info, product identification. */
   if (!read_version_data (h, inf))
     goto lossage;
@@ -308,11 +285,11 @@ pfm_read_dictionary (struct file_handle *h, struct pfm_read_info *inf)
 
  lossage:
   /* Come here on unsuccessful completion. */
-  
+
   fclose (ext->file);
   if (ext && ext->dict)
     free_dictionary (ext->dict);
-  Free (ext);
+  R_Free (ext);
   h->class = NULL;
   h->ext = NULL;
   error(_("error reading portable-file dictionary"));
@@ -320,11 +297,11 @@ pfm_read_dictionary (struct file_handle *h, struct pfm_read_info *inf)
 }
 
 /* Read a floating point value and return its value, or
-   second_lowest_value on error. */
+   NA_REAL on error. */
 static double
 read_float (struct file_handle *h)
 {
-  struct pfm_fhuser_ext *ext = h->ext;			      
+  struct pfm_fhuser_ext *ext = h->ext;
   double num = 0.;
   int got_dot = 0;
   int got_digit = 0;
@@ -379,7 +356,7 @@ read_float (struct file_handle *h)
 
   if (!got_digit)
     lose ((_("Number expected")));
-      
+
   if (ext->cc == 130 /* + */ || ext->cc == 141 /* - */)
     {
       /* Get the exponent.  */
@@ -399,12 +376,12 @@ read_float (struct file_handle *h)
 	}
 
       /* We don't check whether there were actually any digits, but we
-         probably should. */
+	 probably should. */
       if (neg_exp)
 	exp = -exp;
       exponent += exp;
     }
-  
+
   if (!pfm_match (142 /* / */))
     lose ((_("Missing numeric terminator")));
 
@@ -426,28 +403,28 @@ read_float (struct file_handle *h)
 
  overflow:
   if (neg)
-    return -DBL_MAX / 10.;
+    return R_NegInf;
   else
-    return DBL_MAX / 10;
+    return R_PosInf;
 
  lossage:
-  return -DBL_MAX;
+  return NA_REAL;
 }
-  
-/* Read an integer and return its value, or NOT_INT on failure. */
+
+/* Read an integer and return its value, or NA_INTEGER on failure. */
 int
 read_int (struct file_handle *h)
 {
   double f = read_float (h);
 
-  if (f == SECOND_LOWEST_VALUE)
+  if (f == NA_REAL)
     goto lossage;
   if (floor (f) != f || f >= INT_MAX || f <= INT_MIN)
     lose ((_("Bad integer format")));
-  return f;
+  return (int) f;
 
  lossage:
-  return NOT_INT;
+  return NA_INTEGER;
 }
 
 /* Reads a string and returns its value in a static buffer, or NULL on
@@ -459,32 +436,32 @@ read_string (struct file_handle *h)
   struct pfm_fhuser_ext *ext = h->ext;
   static unsigned char *buf;
   int n;
-  
+
   if (h == NULL)
     {
-      Free (buf);
+      R_Free (buf);
       buf = NULL;
       return NULL;
     }
   else if (buf == NULL)
-    buf = Calloc (256, unsigned char);
+    buf = R_Calloc (65536, unsigned char);
 
   n = read_int (h);
-  if (n == NOT_INT)
+  if (n == NA_INTEGER)
     return NULL;
-  if (n < 0 || n > 255)
+  if (n < 0 || n > 65535)
     lose ((_("Bad string length %d"), n));
-  
+
   {
     int i;
 
     for (i = 0; i < n; i++)
       {
-	buf[i] = ext->cc;
+	  buf[i] = (unsigned char) ext->cc;
 	advance ();
       }
   }
-  
+
   buf[n] = 0;
   return buf;
 
@@ -505,7 +482,7 @@ read_header (struct file_handle *h)
     for (i = 0; i < 200; i++)
       advance ();
   }
-  
+
   {
     unsigned char src[256];
     int trans_temp[256];
@@ -526,17 +503,17 @@ read_header (struct file_handle *h)
     for (i = 0; i < 256; i++)
       if (trans_temp[src[i]] == -1)
 	trans_temp[src[i]] = i;
-    
-    ext->trans = Calloc (256, unsigned char);
+
+    ext->trans = R_Calloc (256, unsigned char);
     for (i = 0; i < 256; i++)
-      ext->trans[i] = trans_temp[i] == -1 ? 0 : trans_temp[i];
+	ext->trans[i] = trans_temp[i] == -1 ? 0 : (unsigned char) trans_temp[i];
 
     /* Translate the input buffer. */
     for (i = 0; i < 80; i++)
       ext->buf[i] = ext->trans[ext->buf[i]];
     ext->cc = ext->trans[ext->cc];
   }
-  
+
   {
     unsigned char sig[8] = {92, 89, 92, 92, 89, 88, 91, 93};
     int i;
@@ -568,11 +545,11 @@ read_version_data (struct file_handle *h, struct pfm_read_info *inf)
     static const int map[] = {6, 7, 8, 9, 3, 4, 0, 1};
     char *date = (char *) read_string (h);
     int i;
-    
+
     if (!date)
       return 0;
     if (strlen (date) != 8)
-      lose ((_("Bad date string length %d"), strlen (date)));
+      lose ((_("Bad date string length %lu"), strlen (date)));
     if (date[0] == ' ') /* the first field of date can be ' ' in some
 			   windows versions of SPSS */
 	date[0] = '0';
@@ -589,7 +566,7 @@ read_version_data (struct file_handle *h, struct pfm_read_info *inf)
 	inf->creation_date[10] = 0;
       }
   }
-  
+
   /* Time. */
   {
     static const int map[] = {0, 1, 3, 4, 6, 7};
@@ -599,7 +576,7 @@ read_version_data (struct file_handle *h, struct pfm_read_info *inf)
     if (!time)
       return 0;
     if (strlen (time) != 6)
-      lose ((_("Bad time string length %d"), strlen (time)));
+      lose ((_("Bad time string length %lu"), strlen (time)));
     if (time[0] == ' ') /* the first field of date can be ' ' in some
 			   windows versions of SPSS */
 	time[0] = '0';
@@ -621,12 +598,14 @@ read_version_data (struct file_handle *h, struct pfm_read_info *inf)
   if (pfm_match (65 /* 1 */))
     {
       char *product;
-      
+
       product = (char *) read_string (h);
       if (product == NULL)
 	return 0;
-      if (inf)
-	strncpy (inf->product, product, 61);
+      if (inf) { // placate gcc 8
+	strncpy (inf->product, product, 60);
+	inf->product[60] = '\0';
+      }
     }
   else if (inf)
     inf->product[0] = 0;
@@ -639,13 +618,15 @@ read_version_data (struct file_handle *h, struct pfm_read_info *inf)
       subproduct = (char *) read_string (h);
       if (subproduct == NULL)
 	return 0;
-      if (inf)
-	strncpy (inf->subproduct, subproduct, 61);
+      if (inf) {
+	strncpy (inf->subproduct, subproduct, 60);
+	inf->subproduct[60] = '\0';
+      }
     }
   else if (inf)
     inf->subproduct[0] = 0;
   return 1;
-  
+
  lossage:
   return 0;
 }
@@ -704,20 +685,20 @@ read_variables (struct file_handle *h)
 {
   struct pfm_fhuser_ext *ext = h->ext;
   int i;
-  
+
   if (!pfm_match (68 /* 4 */))
     lose ((_("Expected variable count record")));
-  
+
   ext->nvars = read_int (h);
-  if (ext->nvars <= 0 || ext->nvars == NOT_INT)
+  if (ext->nvars <= 0 || ext->nvars == NA_INTEGER)
     lose ((_("Invalid number of variables %d"), ext->nvars));
-  ext->vars = Calloc (ext->nvars, int);
+  ext->vars = R_Calloc (ext->nvars, int);
 
   /* Purpose of this value is unknown.  It is typically 161. */
   {
     int x = read_int (h);
 
-    if (x == NOT_INT)
+    if (x == NA_INTEGER)
       goto lossage;
 
 /*  According to Akio Sone, there are cases where this is 160 */
@@ -736,7 +717,7 @@ read_variables (struct file_handle *h)
       strcpy (ext->dict->weight_var, name);
       asciify (ext->dict->weight_var);
     }
-  
+
   for (i = 0; i < ext->nvars; i++)
     {
       int width;
@@ -749,19 +730,19 @@ read_variables (struct file_handle *h)
 	lose ((_("Expected variable record")));
 
       width = read_int (h);
-      if (width == NOT_INT)
+      if (width == NA_INTEGER)
 	goto lossage;
       if (width < 0)
 	lose ((_("Invalid variable width %d"), width));
       ext->vars[i] = width;
-      
+
       name = read_string (h);
       if (name == NULL)
 	goto lossage;
       for (j = 0; j < 6; j++)
 	{
 	  fmt[j] = read_int (h);
-	  if (fmt[j] == NOT_INT)
+	  if (fmt[j] == NA_INTEGER)
 	    goto lossage;
 	}
 
@@ -770,11 +751,11 @@ read_variables (struct file_handle *h)
 	 Weirdly enough, there is no # character in the SPSS portable
 	 character set, so we can't check for it. */
       if (strlen ((char *) name) > 8)
-	lose ((_("position %d: Variable name has %u characters"),
+	lose ((_("position %d: Variable name has %lu characters"),
 	       i, strlen ((char *) name)));
       if ((name[0] < 74 /* A */ || name[0] > 125 /* Z */)
 	  && name[0] != 152 /* @ */)
-	lose ((_("position %d: Variable name begins with invalid character"), 
+	lose ((_("position %d: Variable name begins with invalid character"),
 	       i));
       if (name[0] >= 100 /* a */ && name[0] <= 125 /* z */)
 	{
@@ -797,17 +778,17 @@ read_variables (struct file_handle *h)
 	  else if ((c >= 64 /* 0 */ && c <= 99 /* Z */)
 		   || c == 127 /* . */ || c == 152 /* @ */
 		   || c == 136 /* $ */ || c == 146 /* _ */)
-	    name[j] = c;
+	      name[j] = (unsigned char) c;
 	  else
-	    lose ((_("position %d: character `\\%03o' is not valid in a variable name"), 
+	    lose ((_("position %d: character `\\%03o' is not valid in a variable name"),
 		   i, c));
 	}
 
       asciify ((char *) name);
-      if (width < 0 || width > 255)
+      if (width < 0 || width > 65535)
 	lose ((_("Bad width %d for variable %s"), width, name));
 
-      v = create_variable (ext->dict, (char *) name, 
+      v = create_variable (ext->dict, (char *) name,
 			   width ? ALPHA : NUMERIC, width);
       v->get.fv = v->fv;
       if (v == NULL)
@@ -848,7 +829,7 @@ read_variables (struct file_handle *h)
 	      -1, -1, -1,
 	    };
 
-	  static const int map_ofs[MISSING_COUNT] = 
+	  static const int map_ofs[MISSING_COUNT] =
 	    {
 	      -1, 0, 1, 2, -1, -1, -1, 2, 1, 1,
 	    };
@@ -856,7 +837,7 @@ read_variables (struct file_handle *h)
 	  v->miss_type = map_next[v->miss_type];
 	  if (v->miss_type == -1)
 	    lose ((_("Bad missing values for %s"), v->name));
-	  
+
 	  if (map_ofs[v->miss_type] == -1)
 	      error("read_variables : map_ofs[v->miss_type] == -1");
 	  if (!parse_value (h, &v->missing[map_ofs[v->miss_type]], v))
@@ -866,7 +847,7 @@ read_variables (struct file_handle *h)
       if (pfm_match (76 /* C */))
 	{
 	  char *label = (char *) read_string (h);
-	  
+
 	  if (label == NULL)
 	    goto lossage;
 
@@ -895,22 +876,24 @@ parse_value (struct file_handle *h, union value *v, struct variable *vv)
     {
       char *mv = (char *) read_string (h);
       int j;
-      
+
       if (mv == NULL)
 	return 0;
 
-      strncpy ((char *) v->s, mv, 8);
+      // do it this way as strings might not be nul-terminated.
+      // strncpy ((char *) v->s, mv, 8);
+      /* Value labels are always padded with spaces. */
+      memset(v->s, ' ', 8);
       for (j = 0; j < 8; j++)
-	if (v->s[j])
-	  v->s[j] = spss2ascii[v->s[j]];
+	if (mv[j])
+	    v->s[j] = spss2ascii[(unsigned char)mv[j]];
 	else
-	  /* Value labels are always padded with spaces. */
-	  v->s[j] = ' ';
+	  break;
     }
   else
     {
       v->f = read_float (h);
-      if (v->f == SECOND_LOWEST_VALUE)
+      if (v->f == NA_REAL)
 	return 0;
     }
 
@@ -933,10 +916,10 @@ read_value_label (struct file_handle *h)
   int i;
 
   nv = read_int (h);
-  if (nv == NOT_INT)
+  if (nv == NA_INTEGER)
     return 0;
 
-  v = Calloc (nv, struct variable *);
+  v = R_Calloc (nv, struct variable *);
   for (i = 0; i < nv; i++)
     {
       char *name = (char *) read_string (h);
@@ -954,7 +937,7 @@ read_value_label (struct file_handle *h)
     }
 
   n_labels = read_int (h);
-  if (n_labels == NOT_INT)
+  if (n_labels == NA_INTEGER)
     goto lossage;
 
   for (i = 0; i < n_labels; i++)
@@ -964,17 +947,17 @@ read_value_label (struct file_handle *h)
       struct value_label *vl;
 
       int j;
-      
+
       if (!parse_value (h, &val, v[0]))
 	goto lossage;
-      
+
       label = (char *) read_string (h);
       if (label == NULL)
 	goto lossage;
       asciify (label);
 
       /* Create a label. */
-      vl = Calloc (1, struct value_label);
+      vl = R_Calloc (1, struct value_label);
       vl->v = val;
       vl->s = xstrdup (label);
       vl->ref_count = nv;
@@ -988,9 +971,9 @@ read_value_label (struct file_handle *h)
 
 	  /* Create AVL tree if necessary. */
 	  if (!var->val_lab)
-	    var->val_lab = avl_create (val_lab_cmp, (void *) &width);
+	    var->val_lab = R_avl_create (val_lab_cmp, (void *) &width);
 
-	  old = avl_replace (var->val_lab, vl);
+	  old = R_avl_replace (var->val_lab, vl);
 	  if (old == NULL)
 	    continue;
 
@@ -1004,11 +987,11 @@ read_value_label (struct file_handle *h)
 	  free_value_label (old);
 	}
     }
-  Free (v);
+  R_Free (v);
   return 1;
 
  lossage:
-  Free (v);
+  R_Free (v);
   return 0;
 }
 
@@ -1043,16 +1026,16 @@ pfm_read_case (struct file_handle *h, union value *perm, struct dictionary *dict
   /* Check for end of file. */
   if (ext->cc == 99 /* Z */)
     return 0;
-  
+
   /* The first concern is to obtain a full case relative to the data
      file.  (Cases in the data file have no particular relationship to
      cases in the active file.) */
-  tp = temp = Calloc (ext->case_size, union value);
+  tp = temp = R_Calloc (ext->case_size, union value);
   for (tp = temp, i = 0; i < ext->nvars; i++)
     if (ext->vars[i] == 0)
       {
 	tp->f = read_float (h);
-	if (tp->f == SECOND_LOWEST_VALUE)
+	if (tp->f == NA_REAL)
 	  goto unexpected_eof;
 	tp++;
       }
@@ -1062,7 +1045,7 @@ pfm_read_case (struct file_handle *h, union value *perm, struct dictionary *dict
 	if (s == NULL)
 	  goto unexpected_eof;
 	asciify (s);
-	  
+
 	st_bare_pad_copy ((char *) tp->s, s, ext->vars[i]);
 	tp += DIV_RND_UP (ext->vars[i], MAX_SHORT_STRING);
       }
@@ -1075,21 +1058,21 @@ pfm_read_case (struct file_handle *h, union value *perm, struct dictionary *dict
 
       if (v->get.fv == -1)
 	continue;
-      
+
       if (v->type == NUMERIC)
 	perm[v->fv].f = temp[v->get.fv].f;
       else
 	memcpy (perm[v->fv].c, &temp[v->get.fv], v->width);
     }
 
-  Free (temp);
+  R_Free (temp);
   return 1;
 
  unexpected_eof:
   lose ((_("End of file midway through case")));
 
  lossage:
-  Free (temp);
+  R_Free (temp);
   return 0;
 }
 
